define(['jquery', 'rsvp', 'api/shim', 'jquery-actual'], function($, RSVP, shim, actual) {

    var COUNTRY_CODE = (window.navigator.userLanguage || window.navigator.language || '').split('-').pop().toLowerCase();

    var IEVersion = null;
    var tools = {

        prepareConfig: function(data) {
            if(!data.metrics)
                data.metrics = {};

            if(!data.metrics.user_id)
                data.metrics.user_id = null;

            if(!data.metrics.campaign_id)
                data.metrics.campaign_id = null;

            if(!data.social)
                data.social = {};

            if(!data.social.email_from)
                data.social.email_from = null;

            if(!data.social.email_to)
                data.social.email_to = null;

            if(!data.newtab.search.settings)
                data.newtab.search.settings = {};

            if(!data.newtab.search.defaults.quick_links)
                data.newtab.search.defaults.quick_links = [];

            if(!data.newtab.search.settings.quick_links)
                data.newtab.search.settings.quick_links = true;

            if(!data.newtab.wallpapers.defaults.selected)
                data.newtab.wallpapers.defaults.selected = null;

            if(!data.newtab.wallpapers.defaults.favorites)
                data.newtab.wallpapers.defaults.favorites = [];

            if(!data.newtab.wallpapers.defaults.hidden_items)
                data.newtab.wallpapers.defaults.hidden_items = [];

            if(!data.newtab.wallpapers.defaults.library)
                data.newtab.wallpapers.defaults.library = null;

            if(!data.newtab.launcher.defaults.icon_visible)
                data.newtab.launcher.defaults.icon_visible = true;

            if(!data.newtab.wallpapers.defaults.user_source)
                data.newtab.wallpapers.defaults.user_source = null;

            if(!data.newtab.wallpapers.settings)
                data.newtab.wallpapers.settings = {};

            if(!data.newtab.wallpapers.settings.favorites)
                data.newtab.wallpapers.settings.favorites = true;

            if(!data.newtab.wallpapers.settings.hidden_items)
                data.newtab.wallpapers.settings.hidden_items = true;

            if(!data.newtab.wallpapers.settings.library)
                data.newtab.wallpapers.settings.library = true;

            if(!data.newtab.launcher.settings.icon_visible)
                data.newtab.launcher.settings.icon_visible = true;

            if(!data.newtab.launcher.settings.visible)
                data.newtab.launcher.settings.visible = true;

            if(!data.newtab.wallpapers.settings.user_source)
                data.newtab.wallpapers.settings.user_source = true;

            if(!data.newtab.wallpapers.settings.active_tab)
                data.newtab.wallpapers.settings.active_tab = true;

            if(!data.newtab.wallpapers.settings.view)
                data.newtab.wallpapers.settings.view = "stream";

            if(data.newtab.clock.defaults.hour_format === 'auto')
                data.newtab.clock.defaults.hour_format = tools.getLocalHourFormat();

            if(data.newtab.weather.defaults.unit === 'auto')
                data.newtab.weather.defaults.unit = tools.getLocalUnitSystem();

            if(!data.newtab.weather.defaults.history)
                data.newtab.weather.defaults.history = [];

            if(!data.newtab.weather.settings.history)
                data.newtab.weather.settings.history = true;


            if(!data.newtab.user_history.defaults.view)
                data.newtab.user_history.defaults.view = "stream";

            if(!data.newtab.social_share.settings.hasOwnProperty('minimized'))
                data.newtab.social_share.settings.minimized = false;

            if(!data.newtab.shopping) data.newtab.shopping = {};

            if(!data.newtab.shopping.defaults.tab)
                data.newtab.shopping.defaults.tab = "all";

            if(!data.newtab.shopping.settings.tab)
                data.newtab.shopping.settings.tab = "all";

            return data;
        },

        detectIE: function() {
            if(IEVersion !== null) {
                return IEVersion;
            }

            var ua = window.navigator.userAgent;

            var msie = ua.indexOf('MSIE ');
            if (msie > 0) {
                // IE 10 or older => return version number
                IEVersion = parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
                return IEVersion;
            }

            var trident = ua.indexOf('Trident/');
            if (trident > 0) {
                // IE 11 => return version number
                var rv = ua.indexOf('rv:');
                IEVersion = parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
                return IEVersion;
            }

            var edge = ua.indexOf('Edge/');
            if (edge > 0) {
               // Edge (IE 12+) => return version number
               IEVersion = parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
               return IEVersion;
            }

            // other browser
            IEVersion = false;
            return IEVersion;
        },

        getLocalHourFormat: function() {
            return [
                'us', // United States
                'uk', // United Kingdom
                'gb', // Great Britain
                'ph'  // Philippines
            ].indexOf(COUNTRY_CODE) !== -1 ? 'hour-12' : 'hour-24';
        },

        getLocalUnitSystem: function() {
            return [
                'bs', // The Bahamas
                'bz', // Belize
                'ky', // Cayman Islands
                'pw', // Palau
                'us', // United States
                'vi'  // United States Virgin Islands
            ].indexOf(COUNTRY_CODE) !== -1 ? 'imperial' : 'metric';
        },

        formatElapsed: function(delta) {
            var delay = Math.floor(delta / 1000);

            if(delay <= 5)
                return 'just now';

            if(delay < 60)
                return 'less than a minute ago';

            delay = Math.floor(delay / 60);
            if(delay < 60)
                return delay + ' minute' + (delay !== 1 ? 's' : '') + ' ago';

            delay = Math.floor(delay / 60);
            if(delay < 24)
                return delay + ' hour' + (delay !== 1 ? 's' : '') + ' ago';

            delay = Math.floor(delay / 24);
            if(delay < 7)
                return delay + ' day' + (delay !== 1 ? 's' : '') + ' ago';

            delay = Math.floor(delay / 7);
            if(delay < 4)
                return delay + ' week' + (delay !== 1 ? 's' : '') + ' ago';

            delay = Math.floor(delay * 7 / 30.4);
            if(delay < 12)
                return delay + ' month' + (delay !== 1 ? 's' : '') + ' ago';

            delay = Math.floor(delay / 12);
            return delay + ' year' + (delay !== 1 ? 's' : '') + ' ago';
        },

        getType: function(any) {
            return Object.prototype.toString.apply(any).slice(8, -1).toLowerCase();
        },

        isJson: function(any) {
            return tools.getType(any) === 'object';
        },

        isArray: function(any) {
            return tools.getType(any) === 'array';
        },

        mergeJson: function(stronglyTyped, dest /*, ... */) {
            var i = 2, len = arguments.length;

            if(typeof stronglyTyped !== 'boolean') {
                dest = stronglyTyped;
                stronglyTyped = false;
                i = 1;
            }

            for(; i < len; ++i) {
                if(tools.isArray(arguments[i])) {
                    if(typeof dest === 'undefined')
                        dest = [];
                    else if(!tools.isArray(dest)) {
                        // String typing, abort
                        if(stronglyTyped)
                            continue;

                        // Weak typing, override
                        dest = [];
                    }

                    dest = dest.concat(arguments[i]);
                }
                else if(tools.isJson(arguments[i])) {

                    if(typeof dest === 'undefined')
                        dest = {};
                    else if(!tools.isJson(dest)) {
                        // String typing, abort
                        if(stronglyTyped)
                            continue;

                        // Weak typing, override
                        dest = {};
                    }

                    for(var k in arguments[i])
                        dest[k] = tools.mergeJson(stronglyTyped, dest[k], arguments[i][k]);
                }
                else {
                    if(typeof dest === 'undefined')
                        dest = arguments[i];
                    else if(typeof dest === typeof arguments[i]) {
                        if(stronglyTyped)
                            continue;

                        dest = arguments[i];
                    }
                }
            }

            return dest;
        },

        protocolizeUrl: function(url) {
            var protocol = url.match(/^(https?:)?\/\//i);

            if(!protocol)
                return 'http://' + url;

            // No protocol, but // present
            if(protocol[0] === '//')
                return 'http:' + url;

            return url;
        },

        isTopFrame: function() {    //return false when the webpage is running inside an iframe.
            var topFrame;
            var selfFrame;
            try{
                selfFrame = window.self;
                topFrame = window.top;

                if(window.self !== window.top) {
                    return false;
                }
            } catch(ex) { }

            return true;
        },

        openUrl: function(url, newWin) {
            // for branthunder - open speed-dial links in new tab
            if (newWin && tools.isTopFrame()) {
                window.open(url, '_blank');
                return;
            }
            shim.tabs.query({ active: true, lastFocusedWindow: true }, function(err, tabs) {
                if(!err && tabs.length)
                    shim.tabs.update(tabs[0].id, { active: true, url: url });
                else
                    shim.tabs.create({ active: true, url: url });
            });
        },

        openLinkFromEvent: function(e, url) {
            url = url || e.delegateTarget.href;

            // Middle-click + Shift = new tab w/ focus
            if(e.which === 2 && e.shiftKey)
                shim.tabs.create({ active: true, url: url });

            // Middle-click = new tab w/o focus
            else if(e.which === 2)
                shim.tabs.create({ active: false, url: url });

            // Left-click + Shift + Ctrl = new tab w/ focus
            else if(e.which === 1 && e.shiftKey && e.ctrlKey)
                shim.tabs.create({ active: true, url: url });

            // Left-click + Shift = new window w/ focus
            else if(e.which === 1 && e.shiftKey)
                shim.windows.create({ focused: true, url: url });

            // Left-click + Ctrl = new tab w/o focus
            else if(e.which === 1 && e.ctrlKey)
                shim.tabs.create({ active: false, url: url });

            // Left-click = same tab
            else if(e.which === 1)
                tools.openUrl(url);
        },

        openNewtab: function(data, cb) {
            data = data || {};
            var hash = '';

            if(data.defaultWallpaper)
                hash = '#defaultWallpaper';

            shim.runtime.getNewtabInfo(function(newtab) {
                if(data.create)
                    shim.tabs.create({ active: true, url: newtab.url + hash }, cb);
                else
                    tools.tabUpdate({ active: true, url: newtab.url + hash }, cb);
            });
        },

        tabUpdate: function(options, cb) {
            shim.tabs.query({ active: true, lastFocusedWindow: true }, function(tabs) {
                if(tabs.length)
                    shim.tabs.update(tabs[0].id, options, cb);
                else
                    shim.tabs.create(options, cb);
            });
        },

        parsePartnerQueryString: function(str) {
            var partner = {};

            partner.pr = str.match(/(^|\?|&)pr=([^&#]+)/i);
            partner.pr = partner.pr !== null ? partner.pr[2] : null;

            partner.id = str.match(/(^|\?|&)id=([^&#]+)/i);
            partner.id = partner.id !== null ? partner.id[2] : null;

            partner.campaign = str.match(/(^|\?|&)(ent|src)=([^&0-9]*)([0-9]+)/i);
            partner.campaign = partner.campaign !== null ? partner.campaign[4] : null;

            return partner;
        },

        // http://stackoverflow.com/a/16245768
        b64toBlob: function(b64Data, contentType, sliceSize) {
            // Verify browser support
            if(typeof atob !== 'function' || typeof Uint8Array !== 'function' || typeof Blob !== 'function')
                return null;

            contentType = contentType || 'text/plain';
            sliceSize = sliceSize || 512;

            var byteCharacters = atob(b64Data);
            var byteArrays = [];

            for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
                var slice = byteCharacters.slice(offset, offset + sliceSize);

                var byteNumbers = new Array(slice.length);
                for (var i = 0; i < slice.length; i++) {
                    byteNumbers[i] = slice.charCodeAt(i);
                }

                var byteArray = new Uint8Array(byteNumbers);

                byteArrays.push(byteArray);
            }

            return new Blob(byteArrays, { type: contentType });
        },

        dataUriToBlob: function(dataUri) {
            // https://en.wikipedia.org/wiki/Data_URI_scheme#Format
            // Format: data:[<MIME-type>][;charset=<encoding>][;base64],<data>

            var err = function(reason) {
                //console.log('Error in tools.dataUriToBlob: Invalid dataUri - ' + reason, dataUri);
                return null;
            };

            var comma = dataUri.indexOf(',');
            if(comma === -1)
                return err('comma not found');

            var headerParts = dataUri.substr(0, comma).split(';');

            var facts = {}, i, j;
            for(i = 0; i < headerParts.length; ++i) {
                j = headerParts[i].indexOf(/[:=]/);
                if(j === -1)
                    j = headerParts[i].length;

                // Split string properly
                facts[ headerParts[i].substr(0, j) ] = headerParts[i].substr(j + 1);
            }

            var mimeType = facts.data    || 'text/plain';
            var charset  = facts.charset || 'US-ASCII';
            var isBase64 = 'base64' in facts;

            if(!isBase64)
                return err('Only base64 is supported for now');

            return tools.b64toBlob(dataUri.substr(comma + 1), mimeType);
        },

        toggleFullscreen: function (elem, close) {
          elem = elem || document.documentElement;
          if (!document.fullscreenElement && !document.mozFullScreenElement &&
            !document.webkitFullscreenElement && !document.msFullscreenElement) {
            if (close) {
                return;
            }
            if (elem.requestFullscreen) {
              elem.requestFullscreen();
            } else if (elem.msRequestFullscreen) {
              elem.msRequestFullscreen();
            } else if (elem.mozRequestFullScreen) {
              elem.mozRequestFullScreen();
            } else if (elem.webkitRequestFullscreen) {
              elem.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
            }
          } else {
            if (document.exitFullscreen) {
              document.exitFullscreen();
            } else if (document.msExitFullscreen) {
              document.msExitFullscreen();
            } else if (document.mozCancelFullScreen) {
              document.mozCancelFullScreen();
            } else if (document.webkitExitFullscreen) {
              document.webkitExitFullscreen();
            }
          }
        },

        queueImage: function(src) {
            var done = false;
            var deferred = RSVP.defer();

            var img = new Image();
            img.onload  = function() { done = true; deferred.resolve(img); };
            img.onerror = function() { done = true; deferred.reject(null); };

            return {
                promise: deferred.promise,
                load:    function() { if(!done) img.src = src; }
            };
        },

        arrayShuffle: function(a) {
            var j, x, i;
            for (i = a.length; i; i--) {
                j = Math.floor(Math.random() * i);
                x = a[i - 1];
                a[i - 1] = a[j];
                a[j] = x;
            }

            return a;
        },

         //added for search keywords
        shuffle: function (input) {
            if(!input)
                return input;

          for (var i = input.length - 1; i > 0; i--) {
            var randomIndex = Math.floor(Math.random() * (i + 1));
            var itemAtIndex = input[randomIndex];
            input[randomIndex] = input[i];
            input[i] = itemAtIndex;
          }
          return input;
        },

        //use to limit dom element lignes
        uSubstr: function (selector, nb_lignes){

            var elmts = typeof selector=="object"?selector:$(selector), t, h, keywords, t0, text;

            if(!(elmts.length && nb_lignes))
                return false;

            elmts.each(function(){
                var container = $(this);
                t = container.html();
                t0='';

                for(var i=0;i<nb_lignes;i++) t0 += (t0?'<br>':'')+'|';
                container.html(t0);
                h = container.actual('innerHeight');
                keywords = t.split(' ');
                text = '';
                for (i = 0; i < keywords.length; i++){
                  container.html(text+' '+keywords[i]+'...');
                  if(container.actual('innerHeight')>h){
                    container.html(text+'...');
                    break;
                  }
                  else text += ' '+keywords[i];
                }
                if (i >=keywords.length) container.html(text);
            });
        },

        uuid: function (len, radix) {
            var chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.split('');
            var uuid = [], i;
            radix = radix || chars.length;

            if (len) {
              // Compact form
              for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random()*radix];
            } else {
              // rfc4122, version 4 form
              var r;

              // rfc4122 requires these characters
              uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
              uuid[14] = '4';

              // Fill in random data.  At i==19 set the high bits of clock sequence as
              // per rfc4122, sec. 4.1.5
              for (i = 0; i < 36; i++) {
                if (!uuid[i]) {
                  r = 0 | Math.random()*16;
                  uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
                }
              }
            }

            return uuid.join('');
        },

        extractHostname: function (url) {
            var hostname;
            //find & remove protocol (http, ftp, etc.) and get the hostname
            if (url.indexOf("://") > -1) {
                hostname = url.split('/')[2];
            }
            else {
                hostname = url.split('/')[0];
            }

            //find & remove port number
            hostname = hostname.split(':')[0];

            return hostname;
        },

        extractRootDomain: function (url) {
            var _this = this,
                domain = _this.extractHostname(url),
                splitArr = domain.split('.'),
                arrLen = splitArr.length;

            //extracting the root domain here
            if (arrLen > 2) {
                domain = splitArr[arrLen - 2] + '.' + splitArr[arrLen - 1];
            }
            return domain;
        },

        checkSwitchThemes: function(callback) {
            // switch themes options -- start
            var extensionID = 'kgakkbodfgkhiojdmcicoenigbhanpcb';
            $('section[data-my' + 'start' + '-one="new-tab"]').each(function() {
                if(this.getAttribute('data-one-id') == '580e1846a6fde1b55c6ec7de') {
                    shim.runtime.sendMessageToExtension(extensionID, {
                        module: 'newtab-url-change',
                        action: 'check',
                        value: window.location.href
                    }, function(error, info) {
                        callback(!!info);
                    });
                    return;
                }
            });
            // switch themes options -- end
        },
        //Load a wallpaper in cache, then save the informations from this wallpaper to localstorage for quick access via index.html on the next load.
        cacheNextWallpaper: function (info) {
            var url = info.cache_url || info.data_uri || info.image;
            if(!url)
                return;

            $('<img>').attr('src', url).on('load', function() {
                shim.pstorage.set( { 'FASTLOAD_WALLPAPER': { info: info }});
            });
        },
        slugify: function (str) {
            str = str.replace(/^\s+|\s+$/g, '');
            // Make the string lowercase
            str = str.toLowerCase();

            // Remove accents, swap ñ for n, etc
            var from = "ÁÄÂÀÃÅČÇĆĎÉĚËÈÊẼĔȆÍÌÎÏŇÑÓÖÒÔÕØŘŔŠŤÚŮÜÙÛÝŸŽáäâàãåčçćďéěëèêẽĕȇíìîïňñóöòôõøðřŕšťúůüùûýÿžþÞĐđßÆa·/_,:;";
            var to   = "AAAAAACCCDEEEEEEEEIIIINNOOOOOORRSTUUUUUYYZaaaaaacccdeeeeeeeeiiiinnooooooorrstuuuuuyyzbBDdBAa------";
            for (var i=0, l=from.length ; i<l ; i++) {
                str = str.replace(new RegExp(from.charAt(i), 'g'), to.charAt(i));
            }
            // Remove invalid chars
            str = str.replace(/[^a-z0-9 -]/g, '')
            // Collapse whitespace and replace by -
            .replace(/\s+/g, '-')
            // Collapse dashes
            .replace(/-+/g, '-');
            return str;
        },
        hexToRgbA: function (hex, opacity){
            var c;
            if(/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)){
                c= hex.substring(1).split('');
                if(c.length== 3){
                    c= [c[0], c[0], c[1], c[1], c[2], c[2]];
                }
                c= '0x'+c.join('');
                return 'rgba('+[(c>>16)&255, (c>>8)&255, c&255].join(',')+',' + opacity + ')';
            }
            console.log('Bad Hex');
            return '';
        },
        elementInViewport: function (el) {
            var rect = el.getBoundingClientRect();
            return ( rect.top    >= 0 && rect.left   >= 0 && rect.top <= (window.innerHeight || document.documentElement.clientHeight));
        },
        loadHubTab: function(tab,) {

            $('#module-hub ul.tabs > li.' + tab)  
                .addClass('current')
                    .siblings().removeClass('current');

            $('body')
                .removeClass (function (index, css) {
                   return (css.match (/(^|\s)show-hub-tab-\S+/g) || []).join(' ');
                })
                .addClass('show-hub-tab-' + tab + ' hub-tab-' + tab + '-loaded');
        }


    };

    Date.prototype.addHours = function(h) {
       this.setTime(this.getTime() + (h*60*60*1000));
       return this;
    };


    return tools;

});
