define(['jquery', 'lib/js/config', 'knockout', 'json!config-package.json', 'lib/js/tools', 'lib/js/ana_lytics'], function($, config, ko, cfgPkg, tools, analytics) {

    var firstLoad = true;
    var wpSlider = {
            active: false,
            selected: null
        };

    var active_watcher = ko.computed(function() {
        var wp_isActive = config.one.newtab.wallpapers.defaults.hasOwnProperty('visible') ? config.one.newtab.wallpapers.defaults.visible() : true;
        var isActive = wp_isActive &&
                        config.one.newtab.wallpapers.defaults.hasOwnProperty('slide_show') ? config.one.newtab.wallpapers.defaults.slide_show.active() : false;
        $('body').toggleClass('wallpapers-slider-enabled', isActive);

        return isActive;
    });

    var layer_wallpapers_image = $('#layer-wallpapers-image');

    var IDLE_TIMEOUT = 0,
        _idleSecondsCounter = 0,
        running = false,
        timer,
        mouse_Timer,
        slider_timer,
        idleInterval,
        cached_img;


    function CheckIdleTime() {
        if (!active_watcher()) {
            return;
        }

        if (!wpSlider.active) {
            return;
        }

        _idleSecondsCounter++;

        if (_idleSecondsCounter >= timer) {
            nextItem(false, false);
        }
    }

    function nextItem(inverse, cache) {
        var source = config.one.newtab.wallpapers.defaults.user_source() || 'library',
            selected = wpSlider.selected,
            container,
            li,
            img,
            lastItem,
            firstItem,
            url,
            title;

        if ($('body').hasClass('wallpapers-slider-changing')) {
            return;
        }

        if (source === 'preference') {
            source = 'library';
        }

        if (!$('#layer-wallpapers-picker .slide-' + source + ' ul.image-list > li').length) {
            source = 'library';
        }
        container = $('#layer-wallpapers-picker .slide-' + source);

        if (!$('ul.image-list > li', container).length) {
            return;
        }

        lastItem = $('ul.image-list > li', container).last();
        firstItem = $('ul.image-list > li', container).first();

        if (!selected) {
            selected = config.one.newtab.wallpapers.defaults.selected();
        }

        if (cached_img !== undefined) {
            img = cached_img;
            li = $('#' + source + '_' + img.id);
        } else {
            li = $('#' + source + '_' + selected.id);

            if (li.index() === lastItem.index() && !inverse) {
                li = $('ul.image-list > li', container).first();
            } else if (inverse && li.index() === firstItem.index()) {
                li = $('ul.image-list > li', container).last();
            } else {
                li = !inverse ? li.next() : li.prev();
            }
            img = li.data('item');
            if (cache) {
                cached_img = img;
                tools.cacheNextWallpaper({ id: img.id, image: img.image});
                _idleSecondsCounter = timer;
                return;
            }
        }

        url = img.cache_url || img.data_uri || img.image;
        title = img.name || '';

        $('#layer-wallpapers-image-transition').css({
            'background-image': 'url(' + url + ')',
            'background-position': 'center ' + (img.v_align || 'center')
        });
        $('body').addClass('wallpapers-slider-changing');
        $('body').removeClass('wallpapers-slider-changed');

        slider_timer = setTimeout(function() {
            config.one.newtab.wallpapers.defaults.selected(img);

            // save the next item to load
            if (li.index() === lastItem.index() && !inverse) {
                li = $('ul.image-list > li', container).first();
            } else if (inverse && li.index() === firstItem.index()) {
                li = $('ul.image-list > li', container).last();
            } else {
                li = !inverse ? li.next() : li.prev();
            }
            img = li.data('item');
            tools.cacheNextWallpaper({ id: img.id, image: img.image});
            _idleSecondsCounter = 0;

            $('body').removeClass('wallpapers-slider-changing');
            cached_img = undefined;
        }, 1250);

        setTimeout(function() {
            $('.wallpaper-info h3', layer_wallpapers_image).text(title);
            $('body').addClass('wallpapers-slider-changed');
        }, 666);
    }

    function initIde() {

        if (config.one.newtab.wallpapers.defaults.hasOwnProperty('slide_show')) {
            IDLE_TIMEOUT = config.one.newtab.wallpapers.defaults.slide_show.delay();
        }

        if (!IDLE_TIMEOUT) {
            return;
        }

        timer = IDLE_TIMEOUT;
        idleInterval = setInterval(CheckIdleTime, 1000);
    }

    function resetIdle() {
        if (!$('body').hasClass('wallpapers-slider-running')) {
            return;
        }
        wpSlider.active = false;
        $('body').removeClass('wallpapers-slider-running');
        clearTimeout(idleInterval);
        tools.toggleFullscreen(layer_wallpapers_image.get(0));
    }

    function startSlider(ev) {
        var target = $(ev.target || ev.srcElement);

        var selected = config.one.newtab.wallpapers.defaults.selected();
        $('body').addClass('wallpapers-slider-changed');
        $('.wallpaper-info h3', layer_wallpapers_image).text(selected.name || '');
        layer_wallpapers_image.append('<div id="layer-wallpapers-image-transition"></div>');
        initIde();
        tools.toggleFullscreen(layer_wallpapers_image.get(0));
        wpSlider.active = true;
        $('body').toggleClass('wallpapers-slider-running', wpSlider.active);
        _idleSecondsCounter = 0;
        $('body').removeClass('show-wallpapers-picker show-custom-wallpapers');
        $("#module-notsigned-popup").hide();
        analytics.events({ name: 'Action_Wallpaper_SliderShow', label: target.hasClass('play') ?'Play Button':'Button Slideshow' });
    }

    $(document).ready(function() {
        if (!active_watcher()) {
            return;
        }
        //***
        //Silly fix for MYSTNT-834 : Logo Disco Flash
        $('body').addClass('wallpapers-slider-running');
        $('body').removeClass('wallpapers-slider-running');
        //***

        if (document.addEventListener) {
            layer_wallpapers_image.get(0).addEventListener('webkitfullscreenchange', exit_FullScreenHandler, false);
            layer_wallpapers_image.get(0).addEventListener('mozfullscreenchange', exit_FullScreenHandler, false);
            layer_wallpapers_image.get(0).addEventListener('fullscreenchange', exit_FullScreenHandler, false);
            layer_wallpapers_image.get(0).addEventListener('MSFullscreenChange', exit_FullScreenHandler, false);
        }
        layer_wallpapers_image
            .append('<span class="exit_fullscreen"></span>')
            .append('<a class="nav-arrow wp-previous-arrow"></a><a class="nav-arrow wp-next-arrow"></a>')
            .append('<div class="wallpaper-info"><h3></h3></div>');

        $('#layer-wallpapers-image .exit_fullscreen').click(function(ev) {
            resetTimer();
            layer_wallpapers_image.fadeOut(function() {
                $(this).fadeIn();
            });
        });

        layer_wallpapers_image.on('mousemove', function () {
            if (!$('body').hasClass('wallpapers-slider-running')) {
                $(this).removeClass('mouse-moving');
                return;
            }
            $(this).addClass('mouse-moving');
            try {
                clearTimeout(mouse_Timer);
              } catch (e) {}
            mouse_Timer = setTimeout(function () {
                layer_wallpapers_image.removeClass('mouse-moving');
            }, 1000);
        });

        $('body').keydown(function(e) {

            if (!$('body').hasClass('wallpapers-slider-running')) {
                return;
            }

            if ($('body').hasClass('wallpapers-slider-changing')) {
                return;
            }

            try {
                clearTimeout(slider_timer);
            } catch (evt) {}
            if(e.keyCode == 37) { // left
                nextItem(true, true);
            } else if(e.keyCode == 39) { // right
                nextItem(false, true);
            }
        });

        $('#layer-wallpapers-image .nav-arrow').click(function(ev) {
            var target = $(ev.target || ev.srcElement);

            if (!$('body').hasClass('wallpapers-slider-running')) {
                return;
            }

            if ($('body').hasClass('wallpapers-slider-changing')) {
                return;
            }

            try {
                clearTimeout(slider_timer);
            } catch (evt) {}
            if(target.hasClass('wp-previous-arrow')) { // previous
                nextItem(true, true);
            } else { // next
                nextItem(false, true);
            }
        });
    });

    function exit_FullScreenHandler(ev) {
        if (!(document.fullscreenElement || document.mozFullScreenElement || document.webkitFullscreenElement || document.msFullscreenElement)) {
            resetTimer();
            layer_wallpapers_image.fadeOut(function() {
                $(this).fadeIn();
            });
        }
    }

    function resetTimer() {
        if (!active_watcher()) {
            return;
        }
        _idleSecondsCounter = 0;
        wpSlider.active = false;
        $('#layer-wallpapers-image-transition').remove();
        $('body').removeClass('wallpapers-slider-running wallpapers-slider-changing');
        clearTimeout(idleInterval);
        tools.toggleFullscreen(layer_wallpapers_image.get(0), true);
    }

    return {
        startSlider: startSlider
    };

});
